# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['phishnet_api_v3']

package_data = \
{'': ['*']}

install_requires = \
['python-dotenv>=0.10.3,<0.11.0', 'requests>=2.22.0,<3.0.0']

setup_kwargs = {
    'name': 'phishnet-api-v3',
    'version': '0.1.0',
    'description': 'Python3 client to interface with v3 of the api.phish.net REST endpoints.',
    'long_description': '# phishnet_api_v3\n\nphishnet_api_v3 is a Python client for v3 of the [Phish.net API](http://api.phish.net).  It works with either Python 3 and supports all of the endpoints provided by api.phish.net/v3.\n\n# Installation\n\nYou guessed it...\n\n```\npip install phishnet_api_v3\n```\n\n# Getting Started\n\nVersion 3 of the API is a complete rewrite. Access to all endpoints require an apikey. To get one go to https://api.phish.net/keys/ to request one. You can have up to 5 unique apikeys for your applications.\n\nAs you browse the [Phish.net API documentation](https://phishnet.api-docs.io/v3/the-phish-net-api/welcome) you will see that Version 3 also introduced a new security scheme via a unique hash generated on a client to retrieve an authorization key. Like the previous version of our API, there are API methods designated as writable that require a special per-user, per-app authorization key called an "authkey" that must match a uid and your apikey. To obtain this key,  the user you want to access one of these "writeable" endpoints on behalf of must go to https://phish.net/authorize?appid=<<appid>>&uid=<<uid>> to authorize your application. You will need to provide them your application id available at https://api.phish.net (you must be logged in to phish.net )\n\n## Public API Methods\n\nFor API calls that do not require a authkey, you simply need to instantiate the PhishNetAPI class, and call the methods for each of the API methods. There are 2 ways to set the apikey. The easiest way is to create a .env file in the current directory you are calling this API wrapper (see .env.SAMPLE in the root directory of this project.) it should look like this:\n\n```\nexport APIKEY=<<YOURAPIKEYHERE>>\nexport APPID=<<YOURAPPIDHERE>>\nexport PRIVATE_SALT=<<YOURPRIVATESALTHERE>>\n```\nthe PRIVATE_SALT is also available from https://api.phish.net/keys/ and is used to retrieve the authkey from the /authorize/get endpoint. The other way to set the apikey is to pass it into the constructor:\n\n``` python\n>>> from phishnet_api_v3 import PhishNetAPI\n>>> api = PhishNetAPI(<YOURAPIKEYHERE>)\n>>> artists = api.get_artists()\n>>> artists\n{\'error_code\': 0, \'error_message\': None, \'response\': {\'count\': 5, \'data\': {\'1\': {\'artistid\': 1, \'name\': \'Phish\', \'link\': \'http://phish.net/setlists/phish\'}, \'2\': {\'artistid\': 2, \'name\': \'Phish\', \'link\': \'http://phish.net/setlists/trey\'}, \'6\': {\'artistid\': 6, \'name\': \'Phish\', \'link\': \'http://phish.net/setlists/mike\'}, \'7\': {\'artistid\': 7, \'name\': \'Phish\', \'link\': \'http://phish.net/setlists/fish\'}, \'9\': {\'artistid\': 9, \'name\': \'Phish\', \'link\': \'http://phish.net/setlists/page\'}}}}\n```\n\nAttempting to call protected methods without calling authorize(<<uid>>) first will raise `phishnet_api_v3.exceptions.PhishNetAPIError`. If you have your private_salt in .env then authorize() will be called for you. Once you authorize a user then authkey is available in the object for future use. If you change the uid then authorize() will be rerun to get the new authkey. Below is an example of trying to get the authkey of a user who has not authorized your application for his uid:\n\n``` python\n>>> from phishnet_api_v3.api_client import PhishNetAPI\n>>> api = PhishNetAPI()\n>>> api.authorize(1)\nTraceback (most recent call last):\n  File "<stdin>", line 1, in <module>\n  File "/home/fty/git/phishnet_api_v3/phishnet_api_v3/api_client.py", line 89, in authorize\n    self.authkey = self.get_authkey(uid)\n  File "/home/fty/git/phishnet_api_v3/phishnet_api_v3/api_client.py", line 103, in get_authkey\n    return self._authority_get(uid)\n  File "/home/fty/git/phishnet_api_v3/phishnet_api_v3/api_client.py", line 122, in _authority_get\n    response = self.post(function=\'_authority_get\',\n  File "/home/fty/git/phishnet_api_v3/phishnet_api_v3/decorators.py", line 37, in wrapper\n    return f(*args, **kwargs)\n  File "/home/fty/git/phishnet_api_v3/phishnet_api_v3/api_client.py", line 422, in post\n    raise PhishNetAPIError(\nphishnet_api_v3.exceptions.PhishNetAPIError: phish.net API error: authority/get, 3, Invalid authorization key, this application is not writable\n```\n\n## API Methods\n\nStarting with v3 of the phish.net API, all methods are considered "protected" API methods\nand require an API key. You can have this as a OS environment variable via your OS mechanisms or via the .env file or you can pass the API key into the constuctor.  \n\n``` python\n>>> from phishnet_api_v3 import PhishNetAPI\n>>> my_api_key = "<MY API KEY>" # Private API key from http://api.phish.net/keys/\n>>> api = PhishNetAPI(my_api_key)\n>>> api.get_user_details(80)\n{\'error_code\': 0, \'error_message\': None, \'response\': {\'count\': 1, \'data\': {\'80\': {\'uid\': 80, \'username\': \'wsppan\', \'realname\': \'Jay Flaherty\', \'joined\': \'2009-09-29\', \'member_since\': \'10 years ago\', \'avatar\': \'https://smedia.pnet-static.com/avatars/default_avatar.png\', \'link\': \'http://phish.net/user/wsppan\', \'isconfirmed\': \'Yes\', \'bio\': \'My name is Jay and I am a Phishaholic\', \'website\': \'http://\'}}}}\n\n### get_recent_blogs\n```python\nPhishNetAPI.get_recent_blogs(self)\n```\n\nGet a list of recently posted entries to the phish.net blog (last 15 entries).\n:return: json response object of recent blog entries - see tests/data/recent_blogs.json\n\n### get_blogs\n```python\nPhishNetAPI.get_blogs(self, **kwargs)\n```\n\nFind all blogs that match the params below (maximum of 15 results)\n:param **kwargs. Made up of at least one of the the following keys:\n    month: The month of the year as as integer\n    day: The day of the month as as integer\n    year: The year as as integer\n    monthname: month of the year (i.e. January)\n    username: The username of the author\n    author: the uid of the author\n:return: json response object of blogs that match the query(see get_recent_blogs())\n\n### get_artists\n```python\nPhishNetAPI.get_artists(self)\n```\n\nThe artists/all endpoint returns an array of artists whose setlists are tracked\nin the Phish.net setlist database, along with the associated artistid.\n:return: json response object of all artists - see tests/data/all_artists.json\n\n### get_show_attendees\n```python\nPhishNetAPI.get_show_attendees(self, **kwargs)\n```\n\nFind all attendees for a specific show\n:param **kwargs: Made up of at least one of the the following keys:\n    showid: the id for a specific show\n    showdate: The date for a specific show\n:return: json response object of attendees for a specific show.\n    - see tests/data/show_attendees.json\n\n### query_collections\n```python\nPhishNetAPI.query_collections(self, **kwargs)\n```\n\nQuery user collections from the /collections/query endpoint.\n:params **kwargs comprised of the following keys: collectionid, uid, contains.\n    contains is a comma separated string of showid\'s\n:returns: json response object with a list of collections.\n\n### get_collection\n```python\nPhishNetAPI.get_collection(self, collectionid)\n```\n\nGet the details of a collection from the /collections/get endpoint.\n:param collection_id: the collectionid associated with the collection.\n:returns: json object of a collection detail. - see tests/data/get_collection.json\n\n### get_all_jamcharts\n```python\nPhishNetAPI.get_all_jamcharts(self)\n```\n\nGet an array of songs that have jamcharts, song, songid, name, link, and number\nfrom the /jamcharts/all endpoint.\n:return: json response object of all jamcharts - see tests/data/all_jamcharts.json\n\n### get_jamchart\n```python\nPhishNetAPI.get_jamchart(self, songid)\n```\n\nGet the details of a jamchart from the /jamcharts/get endpoint.\n:param song_id: the songid associated with the jamchart.\n:returns: json object of a jamchart detail. - see tests/data/get_jamchart.json\n\n### get_all_people\n```python\nPhishNetAPI.get_all_people(self)\n```\n\nGet a list of personid, name, type, and a link to all shows in which a person is featured\nfrom the people/all endpoint.\n:returns: json object of all people - see tests/data/all_people.json\n\n### get_all_people_types\n```python\nPhishNetAPI.get_all_people_types(self)\n```\n\nGet a dict of all people types from the people/get endpoint.\n:returns: json object of all people types - see tests/data/all_people_types.json\n\n### get_people_by_show\n```python\nPhishNetAPI.get_people_by_show(self, showid)\n```\n\nGet list of performers at a show from the /people/byshow endpoint.\n:param show_id: the showid associated with the show.\n:returns: json object of a list of performers for a show.\n    - see tests/data/people_by_show.json\n\n### get_appearances\n```python\nPhishNetAPI.get_appearances(self, personid, year=None)\n```\n\nGet list of appearances for a particular performer from the /people/appearances endpoint.\n:param person_id: the personid associated with the appearances.\n:param year: optional parameter to narrow the list by year (>=1983)\n:returns: json object of a list of appearances for a performer.\n    - see tests/data/get_appearances.json\n\n### get_relationships\n```python\nPhishNetAPI.get_relationships(self, uid)\n```\n\nReturns an array of friends & fans from the /relationships/get endpoint.\n:param uid: the uid associated with the relationship.\n:returns: json object of a user\'s relationships. - see tests/data/get_relationships.json\n\n### query_reviews\n```python\nPhishNetAPI.query_reviews(self, **kwargs)\n```\n\nQuery user reviews from the /reviews/query endpoint.\n:params **kwargs comprised of the following keys: showid, uid,\n    posted_on, posted_after, posted_before.\n:returns: json response object with a list of reviews.\n\n### get_latest_setlist\n```python\nPhishNetAPI.get_latest_setlist(self)\n```\n\nGet an array with the most recent Phish setlist\n:returns: json object of the latest setlist - see tests/data/latest_setlist.json\n\n### get_setlist\n```python\nPhishNetAPI.get_setlist(self, showid=None, showdate=None)\n```\n\nGet an array with the Phish setlist that matches the showid or showdate.\nIf both parameters are passed in, showid takes precedence. If no params are\npassed in then it returns the latest setlist.\n:returns: json object of matching setlist - see tests/data/latest_setlist.json\n\n### get_recent_setlists\n```python\nPhishNetAPI.get_recent_setlists(self)\n```\n\nGet an array with the 10 most recent Phish setlists\n:returns: json object of the 10 most recent setlist - see tests/data/recent_setlists.json\n\n### get_tiph_setlist\n```python\nPhishNetAPI.get_tiph_setlist(self)\n```\n\nGet an array with a random setlist from today\'s date (MM/DD) in Phish history\n:returns: json object of the setlist - see tests/data/tiph_setlist.json\n\n### get_random_setlist\n```python\nPhishNetAPI.get_random_setlist(self)\n```\n\nGet an array with a random setlist in Phish history\n:returns: json object of the setlist - see tests/data/tiph_setlist.json\n\n### get_in_progress_setlist\n```python\nPhishNetAPI.get_in_progress_setlist(self)\n```\n\nGet the Most Recent Setlist, Including in Progress.\n:returns: json object of the setlist - see tests/data/tiph_setlist.json\n\n### get_show_links\n```python\nPhishNetAPI.get_show_links(self, showid)\n```\n\nGet links associated with a show, including LivePhish links, Phish.net recaps, photos, and more.\n:returns: json object of show links - see tests/data/get_show_links.json\n\n### get_upcoming_shows\n```python\nPhishNetAPI.get_upcoming_shows(self)\n```\n\nGet an array of upcoming shows.\n:returns: json object of upcoming shows - see tests/data/get_upcoming_shows.json\n\n### query_shows\n```python\nPhishNetAPI.query_shows(self, **kwargs)\n```\n\nQuery the show list via one or more parameters. If no paramters are fed,\nreturns error_code 3. Please note that you cannot send an argument of\n"showdate" or "date" to this endpoint. If you know the showdate for which\nyou want details, please use get_setlist()\n\n### get_user_details\n```python\nPhishNetAPI.get_user_details(self, uid)\n```\n\nReturns an array of publically available details about a user. Requires\nan authkey from an authorized user of your application. See authorize().\n:return: json response object with details for a registered phish.net user.\n\n### get_all_venues\n```python\nPhishNetAPI.get_all_venues(self)\n```\n\nReturns an array of venues.\n:return: json response object with all venues - see tests/data/all_venues.json.\n\n### get_venue\n```python\nPhishNetAPI.get_venue(self, venueid)\n```\n\nreturns an array of values, including geographical location about a single venue and,\nif its an alias, the current name.\n:return: json response object with the details abount a venue -- see tests/data/get_venue.json\n\n\n### Methods requiring user authorization\n\nSome protected methods additionally require an authkey to take actions on behalf of specific users.  This includes adding and removing a show to "My Shows". In order to make authorized API calls on \nbehalf of a user, the user needs to be registered with phish.net and they need to go to \nhttps://phish.net/authorize?appid=<<your appid>>&uid=<<their UID>>. Your appid is available from \nhttps://api.phish.net/keys/\n\nOnce those attributes have been set, you can make user-authorized API calls.  For example, lets add, and then remove [11/30/1997](http://phish.net/setlists/?d=1997-11-30) to userid 80\'s list of shows.\n\n``` python\n>>> from phishnet_api_v3.api_client import PhishNetAPI\n>>> api = PhishNetAPI()\n>>> api.authorize(80)\n>>> response = api.query_shows(year=1997, month=11, day=30)\n>>> response[\'response\'][\'data\'][\'showid\']\n>>> response[\'response\'][\'data\'][0][\'showid\']\n1252691618\n>>> api.update_show_attendance(api.uid, showid=1252691618, update=\'add\')\n{"error_code": 0, "error_message": "Successfully added 1997-11-30", "response": {"count": 4, "data": {"action": "add", "showdate": "1997-11-30", "showid": 1252691618, "shows_seen": "140"}}}\n>>> api.update_show_attendance(api.uid, showid=1252691618, update=\'remove\')\n{"error_code": 0, "error_message": "Successfully removed 1997-11-30", "response": {"count": 4, "data": {"action": "remove", "showdate": "1997-11-30", "showid": 1252691618, "shows_seen": "139"}}}\n```\n\n### authorize\n```python\nPhishNetAPI.authorize(self, uid, private_salt=None)\n```\n\nAuthorize the current instance to be able to make privileged API calls on behalf of a selected user.\nIn order to use this method, The phish.net registered user must have gone to\nhttps://phish.net/authorize?appid=X&uid=Y to authorize themselves with your app and allow you to get\nthier authkey via the authority/get endpoint.\n\n:param uid: The uid to authorize on behalf of.\n:param private_salt: The private_salt associated with the apikey. If None, will check\nenvironment variable (set either via the OS environment or .env file.)\n\n',
    'author': 'Jay Flaherty',
    'author_email': 'jayflaherty@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jflaherty/phishnet_api_v3',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
